\echo 'Script version 1.1'


\echo 'Start at' `date`
\qecho 'Start at' `date`

\echo 'Check index statuses'
with mg as (
    select original_table, column_name, rdf_column, rdf_table from bigint_migration_settings where not new_table_migration
)
, pk_index as (
    select i.indisvalid
         , i.indpred is null is_created
         , mg.original_table
         , mg.column_name
       from pg_index i
            join pg_class t on (t.oid = i.indrelid)
       cross join unnest(i.indkey) x
            join pg_attribute a on (a.attrelid = t.oid and a.attnum = x)
            join mg on (t.relname = mg.rdf_table and a.attname = mg.rdf_column)
      where 1=1
        and i.indisunique
        and not i.indisprimary
)
, view_index as (
    select i.indisvalid
         , i.indpred is null is_created
         , mg.original_table
         , mg.column_name
      from pg_index i
           join pg_class t on (t.oid = i.indexrelid)
           join mg on (t.relname = mg.rdf_table || '_view_bigint')

)
select 'PK index on ' || mg.original_table || ' is'
     || case when i.indisvalid is null then ' not exists and will be created'
             when not i.indisvalid then ' INVALID and will be re-created'
             when not i.is_created then ' empty and will be created'
             else ' OK'
        end as "Index status"
  from mg
  left join pk_index i on (i.original_table = mg.original_table and mg.column_name = i.column_name)
union all
select 'VIEW index on ' || mg.original_table || ' is'
     || case when i.indisvalid is null then ' not exists and will be created'
             when not i.indisvalid then ' INVALID and will be re-created'
             when not i.is_created then ' empty and will be created'
             else ' OK'
        end as "Index status"
  from mg
  left join view_index i on (i.original_table = mg.original_table and mg.column_name = i.column_name);

\echo 'Create indexes...'

with dt as (
    select rdf_table
         , column_name
         , rdf_column
         , exists (select 1
                     from bigint_progress_log pl
                    where pl.table_name_from = ms.original_table
                      and pl.column_name = ms.column_name
                      and pl.table_name_to = ms.original_table
                      and pl.done) is_finished
         , exists (select 1
              from pg_index i
                   join pg_class t on (t.oid = i.indexrelid)
             where t.relname = ms.rdf_table || '_view_bigint'
               and i.indpred is null
               and i.indisvalid) view_index_exists
         , exists (select t.relname table_name
                                  , a.attname
                               from pg_index i
                                    join pg_class t on (t.oid = i.indrelid)
                               cross join unnest(i.indkey) x
                                    join pg_attribute a on (a.attrelid = t.oid and a.attnum = x)
                              where 1=1
                                and t.relname = ms.rdf_table
                                and a.attname = ms.rdf_column
                                and i.indisunique
                                and not i.indisprimary
                                and i.indpred is null
                                and i.indisvalid) new_pk_exists
      from bigint_migration_settings ms
     where not ms.new_table_migration
)
select 'drop index if exists "' || rdf_table || '_view_bigint"' from dt where not view_index_exists
union all
select 'create unique index "' || rdf_table || '_view_bigint" on "' || rdf_table || '" (coalesce("' || rdf_column || '", "' || column_name || '"))' from dt where not view_index_exists
union all
select 'drop index if exists "' || column_name || '_PK"' from dt where not new_pk_exists
union all
select 'create unique index "' || column_name || '_PK" on "' || rdf_table || '" ("' || rdf_column || '")' from dt where not new_pk_exists;
\gexec

\echo '... done!'

\echo 'Finish at' `date`
\qecho 'Finish at' `date`